# cornell_test_for_solutions.py

from cornell import *

def test_cornellian_init():
    Cornellian.NEXT_NUM = 1
    flist = ["John","mary","e"]
    llist = ["Smith","doe","o"]
    for i in range(len(flist)):
        f = flist[i]
        l = llist[i]
        c1 = Cornellian(f,l)

        # check this is a Cornellian
        assert isinstance(c1,Cornellian)
        # check attrbitues match
        assert c1.first_name == f, "c1.first_name is " + c1.first_name + \
            " but should be " + f
        assert c1.last_name == l, "c1.last_name is " + c1.last_name + \
            " but should be " + l
        assert c1.cuid == f[0].lower() + l[0].lower() + str(i+1), "c1.id is " + \
            str(c1.cuid) + " but should be " + f[0].lower() + l[0].lower() + str(i+1)
        # check that class_attribute updated
        assert Cornellian.NEXT_NUM == i+2, "Class attribute NEXT_NUM is " + \
            str(Cornellian.NEXT_NUM) + " but should be " + str(i+2)

def test_cornellian_eq():
    c1 = Cornellian("John","Smith")
    c2 = Cornellian("mary","doe")
    c3 = Cornellian("mary","doe")
    c4 = Cornellian("John","Madden")
    c4.cuid = c1.cuid

    assert c1 == c1, "Code shows Cornellians with same ids are not the same"
    assert c2 == c2, "Code shows Cornellians with same ids are not the same"
    assert c3 == c3, "Code shows Cornellians with same ids are not the same"
    assert c4 == c4, "Code shows Cornellians with same ids are not the same"
    assert c2 != c3, "Code shows Cornellians with same name but different ids are the same"
    assert c1 == c4, "Code shows Cornellians with same ids but different names are not the same"
    assert c1 != 5, "Code shows Cornellian is equal to non-Cornellian object"
    assert c1 != "Cornellian", "Code shows Cornellian is equal to non-Cornellian object"

def test_professor_init():
    # Reset Cornellian.NEXT_NUM to make tests work:
    Cornellian.NEXT_NUM = 1
    Professor.departments = []

    # With no starting major
    flist = ["Anne","l","Dr."]
    llist = ["Bracy","lee","Arts"]
    dlist = ["Computer Science", "Computer Science", "Art"]
    for i in range(len(flist)):
        f = flist[i]
        l = llist[i]
        d = dlist[i]
        c1 = Professor(f,l,d)

        # check this is a Cornellian
        assert isinstance(c1,Cornellian)
        # check attrbitues match
        assert c1.first_name == f, "c1.first_name is " + c1.first_name + \
            " but should be " + f
        assert c1.last_name == l, "c1.last_name is " + c1.last_name + \
            " but should be " + l
        assert c1.cuid == f[0].lower() + l[0].lower() + str(i+1), "c1.id is " + \
            str(c1.cuid) + " but should be " + f[0].lower() + l[0].lower() + str(i+1)
        # check that class_attribute updated
        assert Cornellian.NEXT_NUM == i+2, "Class attribute NEXT_NUM is " + \
            str(Cornellian.NEXT_NUM) + " but should be " + str(i+2)
        assert c1.department == dlist[i], "c1.department is " + c1.department + \
            " but should be " + dlist[i]
        assert c1.advisees == [], "c1.advisees is not the empty list"

        if i != 2:
            assert Professor.departments == ["Computer Science"], "Class " + \
                "variable for departments is not correct"
        else:
            assert Professor.departments == ["Computer Science","Art"], "Class " + \
                "variable for departments is not correct"

def test_student_init():
    # Reset Cornellian.NEXT_NUM to make tests work:
    Cornellian.NEXT_NUM = 1

    # With no starting major
    flist = ["John","mary","e"]
    llist = ["Smith","doe","o"]
    for i in range(len(flist)):
        f = flist[i]
        l = llist[i]
        c1 = Student(f,l)

        # check this is a Cornellian
        assert isinstance(c1,Cornellian)
        # check attrbitues match
        assert c1.first_name == f, "c1.first_name is " + c1.first_name + \
            " but should be " + f
        assert c1.last_name == l, "c1.last_name is " + c1.last_name + \
            " but should be " + l
        assert c1.cuid == f[0].lower() + l[0].lower() + str(i+1), "c1.id is " + \
            str(c1.cuid) + " but should be " + f[0].lower() + l[0].lower() + str(i+1)
        # check that class_attribute updated
        assert Cornellian.NEXT_NUM == i+2, "Class attribute NEXT_NUM is " + \
            str(Cornellian.NEXT_NUM) + " but should be " + str(i+2)
        assert c1.college == "", "c1.college is " + c1.college + \
            " but should be the empty string"

    # With starting major and advisors
    Cornellian.NEXT_NUM = 1
    p1 = Professor("Tom","Doctor","Math")
    p2 = Professor("Emily","Tag","History")
    p3 = Professor("Jane","Doe","Computer Science")
    flist = ["John","mary","e"]
    llist = ["Smith","doe","o"]
    clist = ["A and S","CALS",""]
    for i in range(len(flist)):
        f = flist[i]
        l = llist[i]
        c = clist[i]
        c1 = Student(f,l,c)

        # check this is a Cornellian
        assert isinstance(c1,Cornellian)
        # check attrbitues match
        assert c1.first_name == f, "c1.first_name is " + c1.first_name + \
            " but should be " + f
        assert c1.last_name == l, "c1.last_name is " + c1.last_name + \
            " but should be " + l
        assert c1.cuid == f[0].lower() + l[0].lower() + str(i+4), "c1.id is " + \
            str(c1.cuid) + " but should be " + f[0].lower() + l[0].lower() + str(i+4)
        # check that class_attribute updated
        assert Cornellian.NEXT_NUM == i+5, "Class attribute NEXT_NUM is " + \
            str(Cornellian.NEXT_NUM) + " but should be " + str(i+5)
        assert c1.college == c, "c1.college is " + c1.college + \
            " but should be " + c


def test_student_chooseMajor():
    Cornellian.NEXT_NUM = 1
    Professor.departments = []
    Professor.all_of_em = []

    s1 = Student("John","Smith")
    s2 = Student("Mary","Doe")
    s3 = Student("Ezra","Cornell")

    p1 = Professor("Tom","Doctor","Math")
    p2 = Professor("Emily","Tag","History")
    p3 = Professor("Jane","Doe","Computer Science")

    s1.chooseMajor("Math")
    assert s1.major == "Math", "Major is not correct"
    assert s1.advisor == p1, "Advisor is not correct"
    assert s1 in p1.advisees, "Student not in Professor's advisees' list"

    s2.chooseMajor("Computer Science")
    assert s2.major == "Computer Science", "Major is not correct"
    assert s2.advisor == p3, "Advisor is not correct"
    assert s2 in p3.advisees, "Student not in Professor's advisees' list"

    s3.chooseMajor("Art")
    assert s3.major == "Art", "Major is not correct"
    assert s3.advisor == None, "Advisor is not correct"



# Script will run when file is called
print("Start testing")
print()

print("Testing Cornellian.__init__")
test_cornellian_init()
print("Testing Cornellian.__eq__")
test_cornellian_eq()

print("Testing Professor.__init__")
test_professor_init()

print("Testing Student.__init__")
test_student_init()
print("Testing Student.chooseMajor")
test_student_chooseMajor()

print()
print("Passed all tests!!!")
